# https://brainsnacks.org/decisiontree/を基に
# https://qiita.com/renesisu727/items/844648d6c60e578ce944の一部を加えた
# decision_tree_breastcancer.py
# 乳癌の良性、悪性を30の特徴量から決定木手法で判定する

#　必要なライブラリの読み込み
%matplotlib inline
import numpy as np
import matplotlib.pyplot as plt
#import matplotlib.color import ListedColormap
import pandas as pd
import graphviz

#　機械学習モデル
from sklearn.tree import DecisionTreeClassifier
from sklearn.tree import export_graphviz
from sklearn.model_selection import train_test_split

#　予測値の評価用　
from sklearn.metrics import accuracy_score
from sklearn.metrics import classification_report

#sklean cancer dataの読み込み
from sklearn.datasets import load_breast_cancer

dataset = load_breast_cancer()

# 特徴量 (説明変数)
X = pd.DataFrame(dataset.data,
                 columns=dataset.feature_names)
#ラベル（目的変数)
y = pd.Series(dataset.target, name='y')

# データ詳細確認
print('X shape: (%i,%i)' %X.shape)
print(y.value_counts())
display(X.join(y).head())

# ホールドアウト
X_train,X_test,y_train,y_test = train_test_split(X,y,test_size=0.20,random_state=0,stratify=y)
Column_List_train = X_train.columns

#ハイパーパラメータの調節は無し
tree = DecisionTreeClassifier(random_state=0,max_depth=3)
tree.fit(X_train,y_train)

print("Decision_Tree_Cross-validation train_scores: {:.2f}".format(tree.score(X_train,y_train)))
print("Decision_Tree_Cross-validation test_scores: {:.2f}".format(tree.score(X_test,y_test)))

# 決定木の可視化
dot_data = export_graphviz(tree)
graph = graphviz.Source(dot_data)
graph.render('breast-cancer-tree', format='png')

# 特徴量の重要度の棒グラフ
feature = tree.feature_importances_
label = X.columns
indices = np.argsort(feature)

fig =plt.figure (figsize = (10,10))

plt.barh(range(len(feature)), feature[indices])

plt.yticks(range(len(feature)), label[indices], fontsize=14)
plt.xticks(fontsize=14)
plt.ylabel("Feature", fontsize=18)
plt.xlabel("Feature Importance", fontsize=18)