# https://qiita.com/kuro_take/items/b753ff5daf03aed20682
# を参考に自分のPCからgoogle colabで動くように変更しました(2022_11_5)
# 漫画の名言はhttps://ciatr.jp/topics/317301から引用しました
# 「Transformer」は2017年にgoogleが発表した深層学習を用いた自然言語処理ソフト
# で感情分析、質疑応答、言語モデル、テキスト生成、要約、固有表現抽出、翻訳ができる
# sentiment_analysis.py

!pip install transformers
!pip install sentencepiece

from transformers import pipeline 
from transformers import AutoModelForSequenceClassification 
from transformers import BertJapaneseTokenizer 

# パイプラインの準備
model = AutoModelForSequenceClassification.from_pretrained('daigo/bert-base-japanese-sentiment') 
tokenizer = BertJapaneseTokenizer.from_pretrained('cl-tohoku/bert-base-japanese-whole-word-masking') 
nlp = pipeline("sentiment-analysis", model=model, tokenizer=tokenizer) 

# マンガの台詞
manga_list = ["生殺与奪の権を他人に握らせるな！",
              "俺に勝てるのは俺だけだ",
              "強い言葉を使うなよ、弱く見えるぞ",
              "我が生涯に一片の悔いなし！",
              "1位と最下位との差なんて大したことねーんだよ ゴールすることとしないことの差に比べりゃ",
              "未来なんて、ちょっとしたはずみで、どんどん変わるから",
              "他人にやらされてた練習を努力とは言わねえだろ",
              "今の時代を作れるのは、今を生きてる人間だけだよ",
              "努力した者が全て報われるとは限らん",
              "成功した者は皆すべからく努力しておる"]

# 感情分析の実行と出力
for manga in manga_list:
    print(nlp(manga))

# 以下が出力結果です
#[{'label': 'ポジティブ', 'score': 0.6454975008964539}]
#[{'label': 'ポジティブ', 'score': 0.8803699016571045}]
#[{'label': 'ネガティブ', 'score': 0.5186553597450256}]
#[{'label': 'ポジティブ', 'score': 0.6479748487472534}]
#[{'label': 'ポジティブ', 'score': 0.7210435271263123}]
#[{'label': 'ネガティブ', 'score': 0.7709633111953735}]
#[{'label': 'ネガティブ', 'score': 0.786268413066864}]
#[{'label': 'ポジティブ', 'score': 0.9414372444152832}]
#[{'label': 'ネガティブ', 'score': 0.7014599442481995}]
#[{'label': 'ポジティブ', 'score': 0.9474733471870422}]
